package flexible_employment

import (
	"applet/app/utils"
	"crypto/md5"
	"encoding/hex"
	"encoding/json"
	"fmt"
	"io/ioutil"
	"net/http"
	"net/url"
	"sort"
	"strings"
	"time"
)

type PuPiaoService struct {
	BaseURL     string
	MerchantNo  string
	AgreementId string
	AppKey      string
	SecretKey   string
}

const NewUrlForPrd = "http://merchant.linglong.net.cn"
const NewUrlForDev = "http://saas-merchant-test.youjiayun.com"

func NewNewPuPiao(appKey, secretKey, merchantNo, agreementId string) *PuPiaoService {
	var baseURL string
	if true {
		baseURL = NewUrlForPrd
	} else {
		baseURL = NewUrlForDev
	}
	return &PuPiaoService{
		BaseURL:     baseURL,
		MerchantNo:  merchantNo,
		AgreementId: agreementId,
		AppKey:      appKey,
		SecretKey:   secretKey,
	}
}

func (as *PuPiaoService) Encrypt(data map[string]interface{}) string {
	utils.FilePutContents("new_pupiao_encrypt", utils.SerializeStr(data))
	// 排序键
	keys := make([]string, 0, len(data))
	for k := range data {
		keys = append(keys, k)
	}
	sort.Strings(keys)

	var str = as.SecretKey
	for _, k := range keys {
		v := data[k]
		if k == "payDetailList" {
			strByte, _ := json.Marshal(v)
			var tmpMap []map[string]string
			json.Unmarshal(strByte, &tmpMap)
			str += fmt.Sprintf("[{\"detailOrderNo\":\"%s\",\"amount\":\"%s\",\"phone\":\"%s\",\"idCard\":\"%s\",\"accountNo\":\"%s\",\"userName\":\"%s\"}]",
				tmpMap[0]["detailOrderNo"],
				tmpMap[0]["amount"],
				tmpMap[0]["phone"],
				tmpMap[0]["idCard"],
				tmpMap[0]["accountNo"],
				tmpMap[0]["userName"],
			)
		} else {
			toString, _ := utils.InterfaceToString(v)
			str += toString
		}
	}

	return utils.MD5ToLower32(str)
}

func (as *PuPiaoService) CurlPost(url string, data map[string]interface{}) (map[string]interface{}, error) {
	sign := as.Encrypt(data)

	jsonData, err := json.Marshal(data)
	if err != nil {
		return nil, err
	}

	req, err := http.NewRequest("POST", url, strings.NewReader(string(jsonData)))
	if err != nil {
		return nil, err
	}

	req.Header.Set("Content-Type", "application/json; charset=utf-8")
	req.Header.Set("orderId", fmt.Sprintf("%d%d", time.Now().Unix(), time.Now().Nanosecond()))
	req.Header.Set("merchantNo", as.MerchantNo)
	req.Header.Set("agreementId", as.AgreementId)
	req.Header.Set("sign", sign)
	req.Header.Set("appKey", as.AppKey)
	req.Header.Set("requireTime", time.Now().Format("2006-01-02 15:04:05"))

	client := &http.Client{}
	resp, err := client.Do(req)
	if err != nil {
		return nil, err
	}
	defer resp.Body.Close()

	body, err := ioutil.ReadAll(resp.Body)
	if err != nil {
		return nil, err
	}

	var result map[string]interface{}
	err = json.Unmarshal(body, &result)
	if err != nil {
		return nil, err
	}

	return result, nil
}

func (as *PuPiaoService) EncryptGet(data url.Values) string {
	// 将 data 转换为 sorted keys 字符串
	var keys []string
	for k := range data {
		keys = append(keys, k)
	}
	sort.Strings(keys)
	var str strings.Builder
	str.WriteString(as.SecretKey)
	for _, k := range keys {
		values := data[k]
		// 如果 key 有多个值，这里简单处理为只取第一个值
		if len(values) > 0 {
			str.WriteString(values[0])
		}
	}
	hasher := md5.New()
	hasher.Write([]byte(str.String()))
	return hex.EncodeToString(hasher.Sum(nil))

}

func (as *PuPiaoService) CurlGet(uri string, data url.Values) (map[string]interface{}, error) {
	sign := as.EncryptGet(data)
	// 附加查询参数到 URL
	query := url.Values{}
	for k, v := range data {
		query.Add(k, v[0]) // 假设每个 key 只有一个值
	}
	query.Add("sign", sign)
	fullURL := uri + "?" + query.Encode()
	req, err := http.NewRequest("GET", fullURL, nil)
	if err != nil {
		return nil, err
	}

	req.Header.Set("orderId", fmt.Sprintf("%d", time.Now().Unix()))
	req.Header.Set("merchantNo", as.MerchantNo)
	req.Header.Set("agreementId", as.AgreementId)
	req.Header.Set("appKey", as.AppKey)
	req.Header.Set("requireTime", time.Now().Format("2006-01-02 15:04:05"))
	client := &http.Client{}
	resp, err := client.Do(req)
	if err != nil {
		return nil, err
	}
	defer resp.Body.Close()
	body, err := ioutil.ReadAll(resp.Body)
	if err != nil {
		return nil, err
	}

	var result map[string]interface{}
	err = json.Unmarshal(body, &result)
	if err != nil {
		return nil, err
	}
	return result, nil
}
